using System;
using System.Collections.Generic;
using System.Text;

using Perception.FormulaDatabase.Functions;
using Perception.FormulaDatabase.FunctionSupport;
using RecordingInterface;

namespace FormulaDBFunctions
{
    public class MyFunctionInfo : IFunctionInfo
    {
        #region IFunctionInfo Members

        public string Category
        {
            get
            {
                return "CSI Functions";
            }
        }

        public IFunction CreateFunction()
        {
            // for testing purposes as long as the function
            // is not implemented: return null;
            return new CreateWaveformFromAverage();
        }

        public string Description
        {
            get
            {
                return "Creates a DC waveform with value of average of the waveform to process (InWave).";
            }
        }

        public string Example
        {
            get
            {
                return "@CSI_MeanWave(Active.Group1.Recorder1.Ch_A2)";
            }
        }

        public int MinimumParameterCount
        {
            get
            {
                return 1;
            }
        }

        public string Name
        {
            get
            {
                return "CSI_MeanWave";
            }
        }

        public string[] ParameterDescriptions
        {
            get
            {
                string[] ParamDescriptions = { "The waveform to process" };
                return ParamDescriptions;
            }
        }

        public string[] ParameterNames
        {
            get
            {
                string[] ParamNames = { "InWave" };
                return ParamNames;
            }
        }

        public DataSrcManager.PoolEntryType[] ParameterTypes
        {
            get
            {
                DataSrcManager.PoolEntryType[] PETypes = { DataSrcManager.PoolEntryType.PoolEntryType_Waveform };
                return PETypes;
            }
        }

        #endregion
    }
    
    public class CreateWaveformFromAverage : IFunction, IDataSrc
    {
        #region IFunction Members

        public void Init(IDataSrc[] Parameters, out IDataSrc Result)
        {
            Result = this;
        }

        public PoolEventType OnDataAdded(double StartTime, double EndTime)
        {
            return PoolEventType.DataAdded;
        }

        public void Reset()
        {
        }

        #endregion

        #region IDataSrc Members

        public void Data(double StartTime, double EndTime, out object Data)
        {
            Data = null;
        }

        public DataSourceDataType DataType
        {
            get { return DataSourceDataType.DataSourceDataType_AnalogWaveform; }
        }

        public void GetUTCTime(out int Year, out int YearDay, out double Time, out bool Valid)
        {
            Year = 2006;
            YearDay = 10;
            Time = 1;
            Valid = true;
        }

        public object GetValueAtTime(double Time)
        {
            return double.NaN;
        }

        public string Name
        {
            get { return ""; }
        }

        public IProperties Properties
        {
            get { return null; }
        }

        public DataSourceStatus Status
        {
            get { return DataSourceStatus.DataSourceStatus_Static; }
        }

        public IDataSweeps Sweeps
        {
            get { return null; }
        }

        public DataSourceTimeInfo TimeInfo
        {
            get { return DataSourceTimeInfo.DataSourceTimeInfo_Implicit; }
        }

        public object Value
        {
            get { return null; }
        }

        public string XUnit
        {
            get { return ""; }
        }

        public string YUnit
        {
            get { return ""; }
        }

        #endregion
    }
}
